@@ -23,14 +23,13 @@ output is for non-PDL expressions. To that end, small PDLs (up to 8
 elements) are stored as inline perl expressions, midsized PDLs (up to
 200 elements) are stored as perl expressions above the main data
 structure, and large PDLs are stored as FITS files that are uuencoded
-and included in the dump string. (You have to have access to uuencode(1)
-for this to work).
+and included in the dump string. (You have to have access to either 
+uuencode(1) or the CPAN module Convert::UU for this to work).
 
 No attempt is made to shrink the output string -- for example, inlined
 PDL expressions all include explicit reshape() and typecast commands,
-and uuencode() is notoriously inefficient.  So your data structures
-will grow when you dump them.  Gzip will shrink the output by
-about a factor of 10 for typical small data structures.
+and uuencoding expands stuff by a factor of about 1.5.  So your data
+structures will grow when you dump them. 
 
 =head1 Bugs
 
@@ -79,6 +78,9 @@ This package comes with NO WARRANTY.
 =item * 1.3 (15-May-2002): Added checking for tied objects in gethdr()
   [workaround for hole in Data::Dumper]
 
+=item * 1.4 (15-Jan-2003): Added support for Convert::UU as well as
+  command-line uu{en|de}code
+
 =back
 
 =head1 FUNCTIONS
@@ -94,17 +96,31 @@ BEGIN{
 
   package PDL::IO::Dumper;
 
-  $PDL::IO::Dumper::VERSION = 1.3.1;
+  $PDL::IO::Dumper::VERSION = '1.3.1';
   
   @PDL::IO::Dumper::ISA = ( Exporter ) ;
   @PDL::IO::Dumper::EXPORT_OK = qw( fdump sdump frestore deep_copy);
   @PDL::IO::Dumper::EXPORT = @EXPORT_OK;
   %PDL::IO::Dumper::EXPORT_TAGS = ( Func=>[@EXPORT_OK]);
 
+  eval "use Convert::UU;";
+  $PDL::IO::Dumper::convert_ok = !$@;
+
+  my $a = sub {
+      my($prog) = 'uudecode';
+      my $pathsep = $^O =~ /win32/i ? ';' : ':';
+      my $exe = $^O =~ /win32/i ? '.exe' : '';
+      for(split $pathsep,$ENV{PATH}){return 1 if -x "$_/$prog$exe"}
+      return 0;
+  };
+  $PDL::IO::Dumper::uudecode_ok = &$a('uudecode');
+
   use PDL;
   use PDL::Exporter;
+  use PDL::Config;
   use Data::Dumper;
   use Carp;
+  use IO::File;
 }
 
 ######################################################################
@@ -125,7 +141,7 @@ Dump a data structure to a string.
 =for description
 
 sdump dumps a single complex data structure into a string.  You restore
-the data structure by eval'ing the string.  Since eval is a builtin, no
+the data structure by eval-ing the string.  Since eval is a builtin, no
 convenience routine exists to use it.
 
 =cut
@@ -143,7 +159,6 @@ sub PDL::IO::Dumper::sdump {
   my($v);
   my($dups);
   foreach $v(keys %pdls) {
-##    print STDERR "Hey! $v->".(defined $pdls{$v} ? $pdls{$v} : "[empty]" )."!\n";
     $dups++ if($pdls{$v} >1);
   }
   print STDERR "Warning: duplicated PDL ref.  If sdump hangs, you have a circular reference.\n"  if($dups);
@@ -175,27 +190,28 @@ Dump a data structure to a file
 =for description
 
 fdump dumps a single complex data structure to a file.  You restore the
-data structure by eval'ing the perl code put in the file.  A convenience
+data structure by eval-ing the perl code put in the file.  A convenience
 routine (frestore) exists to do it for you.
 
 I suggest using the extension '.pld' or (for non-broken OS's) '.pdld'
-to distinguish Dumper files.  That way they're reminiscent of .pl
+to distinguish Dumper files.  That way they are reminiscent of .pl
 files for perl, while still looking a little different so you can pick
 them out.  You can certainly feed a dump file straight into perl (for
-syntax checking) but it won't do much for you, just build your data
+syntax checking) but it will not do much for you, just build your data
 structure and exit.
 
 =cut
 
 sub PDL::IO::Dumper::fdump { 
   my($struct,$file) = @_;
-  if(!open(FDuMPFILE,">$file")) {
-    Carp::cluck ("fdump: couldn't open '$file'\n");
-     return undef;
+  my $fh = IO::File->new( ">$file" );
+  unless ( defined $fh ) {
+      Carp::cluck ("fdump: couldn't open '$file'\n");
+      return undef;
   }
-  print FDuMPFILE "####################\n## PDL::IO::Dumper dump file -- eval this in perl/PDL.\n\n";
-  print FDuMPFILE sdump($struct);
-  close FDuMPFILE;
+  $fh->print( "####################\n## PDL::IO::Dumper dump file -- eval this in perl/PDL.\n\n" );
+  $fh->print( sdump($struct) );
+  $fh->close();
   return $struct;
 }
 
@@ -225,13 +241,17 @@ sub PDL::IO::Dumper::frestore {
   local($_);
   my($fname) = shift;
 
-  if(!open(fREsTOrE,"<$fname")){
+  my $fh = IO::File->new( "<$fname" );
+  unless ( defined $fh ) {
     Carp::cluck("frestore:  couldn't open '$file'\n");
-      return undef;
-    }
+    return undef;
+  }
 
-  my($file) = join("",<fREsTOrE>);
-  eval $file;
+  my($file) = join("",<$fh>);
+  
+  $fh->close;
+
+  return eval $file;
 }
 
 ######################################################################
@@ -269,7 +289,7 @@ files and are always treated as huge, regardless of size.
 =cut
 
 $PDL::IO::Dumper::small_thresh = 8;   # Smaller than this gets inlined
-$PDL::IO::Dumper::med_thresh   = 200; # Smaller than this gets eval'ed
+$PDL::IO::Dumper::med_thresh   = 400; # Smaller than this gets eval'ed
                                       # Any bigger gets uuencoded
 
 sub PDL::IO::Dumper::big_PDL {
@@ -366,6 +386,64 @@ sub PDL::IO::Dumper::stringify_PDL{
 
 ######################################################################
 
+=head2 PDL::IO::Dumper::uudecode_PDL
+
+=for ref
+
+Recover a PDL from a uuencoded string [Internal routine]
+
+This routine encapsulates uudecoding of the dumped string for large piddles. 
+It's separate to encapsulate the decision about which method of uudecoding
+to try (both the built-in Convert::UU and the shell command uudecode(1) 
+are supported).
+
+=cut
+
+# should we use OS/library-level routines for creating
+# a temporary filename?
+#
+sub _make_tmpname () {
+    # should we use File::Spec routines to create the file name?
+    return $PDL::Config{TEMPDIR} . "/tmp-$$.fits";
+}
+
+# For uudecode_PDL:
+#
+# uudecode on OS-X needs the -s option otherwise it strips off the
+# path of the data file - which messes things up. We could change the
+# logic so that we explicitly tell uudecode where to create the output
+# file, except that this is also OS-dependent (-o <filename> on OS-X/linux,
+# -p on solaris/OS-X to write to stdout, any others?),
+# so we go this way for now as it is less-likely to break things
+#
+my $uudecode_string = "|uudecode";
+$uudecode_string .= " -s" if $^O =~ m/darwin|((free|open|net)bsd)/;
+
+sub PDL::IO::Dumper::uudecode_PDL {
+    my $lines = shift;
+    my $out;
+    my $fname = _make_tmpname();
+    if($PDL::IO::Dumper::uudecode_ok) {
+	my $fh = IO::File->new( $uudecode_string );
+	$lines =~ s/^[^\n]*\n/begin 664 $fname\n/o;
+	$fh->print( $lines );
+	$fh->close;
+    }
+    elsif($PDL::IO::Dumper::convert_ok) {
+	my $fh = IO::File->new(">$fname");
+	my $fits = Convert::UU::uudecode($lines);
+	$fh->print( $fits );
+	$fh->close();
+    } else {
+      barf("Need either uudecode(1) or Convert::UU to decode dumped PDL.\n");
+  }
+
+  $out = rfits($fname);
+  unlink($fname);
+
+  $out;
+}
+ 
 =head2 PDL::IO::Dumper::dump_PDL
 
 =for ref
@@ -407,22 +485,34 @@ sub PDL::IO::Dumper::dump_PDL {
     else { 
       
       ##
-      ## Write FITS file, uuencode it, snarf it up, and clean up /tmp
+      ## Write FITS file, uuencode it, snarf it up, and clean up the
+      ## temporary directory
       ##
-      my($fname) = "/tmp/$$.fits";
+      my $fname = _make_tmpname();
       wfits($_,$fname);
-      open(FITSFILE,"uuencode $fname $fname |");
-      my(@uulines) = <FITSFILE>;
+      my(@uulines);
+
+      if($PDL::IO::Dumper::uudecode_ok) {
+	  my $fh = IO::File->new( "uuencode $fname $fname |" );
+	  @uulines = <$fh>;
+	  $fh->close;
+    } elsif($PDL::IO::Dumper::convert_ok) {
+	# Convert::UU::uuencode does not accept IO::File handles
+        # (at least in version 0.52 of the module)
+	#
+	open(FITSFILE,"<$fname");
+	@uulines = ( Convert::UU::uuencode(*FITSFILE) );
+    } else {
+	barf("dump_PDL: Requires either uuencode or Convert:UU");
+    }
       unlink $fname;
       
       ## 
       ## Generate commands to uudecode the FITS file and resnarf it
       ##
-      @s = ("open DuMPERFILE,'|(cd /tmp; uudecode)'; print DuMPERFILE <<'blat'\n",
+      @s = ("my(\$$pdlid) = PDL::IO::Dumper::uudecode_PDL(<<'DuMPERFILE'\n",
 	    @uulines,
-	    "blat\n;\n",
-	    "close DuMPERFILE;\n",
-	    "my(\$$pdlid) = rfits('$fname'); unlink '$fname';\n",
+	    "\nDuMPERFILE\n);\n",
 	    "\$$pdlid->hdrcpy(".$_->hdrcpy().");\n"
 	    );
 
@@ -450,7 +540,6 @@ sub PDL::IO::Dumper::dump_PDL {
     ## Ultimately, Data::Dumper will get fixed to handle tied objects, 
     ## and this kludge will go away.
     ## 
-#    print "hdr: ",$_->hdr()," keys:",scalar(keys(%{$_->hdr()})),"\n";
 
     if( scalar(keys %{$_->hdr()}) ) {
       if( ((tied %{$_->hdr()}) || '') =~ m/Astro::FITS::Header\=/) {
